# -*- coding: utf-8 -*-
"""Revelation root module with package info"""

from .app import PresentationReloader, Revelation

__author__ = "Humberto Rocha"
__email__ = "humrochagf@gmail.com"
__version__ = "0.5.0"
__all__ = ["PresentationReloader", "Revelation"]



# -*- coding: utf-8 -*-
"""Cli tool to handle revelation commands"""

import os
import shutil
from functools import partial

import click
from geventwebsocket import Resource, WebSocketServer
from werkzeug.debug import DebuggedApplication

import revelation
from revelation import Revelation, PresentationReloader
from revelation.utils import (
    download_reveal,
    extract_file,
    make_presentation,
    move_and_replace,
)


REVEALJS_FOLDER = os.path.join(
    os.path.join(os.path.dirname(revelation.__file__), "static"), "revealjs"
)

# DRY form for echoing errors
error_echo = partial(click.secho, err=True, fg="red", bold=True)


@click.group(invoke_without_command=True)
@click.option("--version", "-v", is_flag=True, default=False)
@click.pass_context
def cli(ctx, version):
    """Base command function, it gets the context and passes it to
    its subcommands"""
    if not ctx.invoked_subcommand and version:
        click.echo(revelation.__version__)
        ctx.exit()
    elif not ctx.invoked_subcommand:
        click.echo(ctx.get_help())


@cli.command("installreveal", help="Install or upgrade reveal.js dependency")
@click.option("--url", "-u", help="Reveal.js download url")
def installreveal(url):
    """Reveal.js installation command

    Receives the download url to install from a specific version or
    downloads the latest version if noting is passed
    """
    click.echo("Downloading reveal.js...")

    download = download_reveal(url)

    click.echo("Installing reveal.js...")

    move_and_replace(extract_file(download[0]), REVEALJS_FOLDER)

    click.echo("Installation completed!")


@cli.command("mkpresentation", help="Create a new revelation presentation")
@click.argument("presentation")
@click.pass_context
def mkpresentation(ctx, presentation):
    """Make presentation project boilerplate"""
    if os.path.exists(presentation):
        error_echo("Error: '{}' already exists.".format(presentation))
        ctx.exit(1)

    click.echo("Starting a new presentation...")

    make_presentation(presentation)


@cli.command("mkstatic", help="Make a static presentation")
@click.argument("presentation", default=os.getcwd())
@click.option("--config", "-c", default=None, help="Custom configuration file")
@click.option("--media", "-m", default=None, help="Custom media folder")
@click.option("--theme", "-t", default=None, help="Custom theme folder")
@click.option(
    "--style-override-file",
    "-s",
    "style",
    default=None,
    help="Custom css file to override reveal.js styles",
)
@click.option(
    "--output-folder",
    "-o",
    default="output",
    help="Folder where the static presentation will be generated",
)
@click.option(
    "--output-file",
    "-f",
    default="index.html",
    help="File name of the static presentation",
)
@click.option(
    "--force", "-r", is_flag=True, help="Overwrite the output folder if exists"
)
@click.pass_context
def mkstatic(
    ctx,
    presentation,
    config,
    media,
    theme,
    output_folder,
    output_file,
    force,
    style,
):
    """Make static presentation"""

    # Check if reveal.js is installed
    if not os.path.exists(REVEALJS_FOLDER):
        click.echo("Reveal.js not found, running installation...")
        ctx.invoke(installreveal)

    output_folder = os.path.realpath(output_folder)

    # Check for style override file
    if os.path.isfile(output_folder):
        error_echo(
            "Error: '{}' already exists and is a file.".format(output_folder)
        )
        ctx.exit(1)

    # Check for presentation file
    if os.path.isfile(presentation):
        path = os.path.dirname(presentation)
    else:
        error_echo("Error: Presentation file not found.")
        ctx.exit(1)

    if style and (not os.path.isfile(style) or not style.endswith(".css")):
        click.echo("Error: Style is not a css file or does not exists.")
        ctx.exit(1)

    if os.path.isdir(output_folder):
        if force:
            shutil.rmtree(output_folder)
        else:
            error_echo(
                (
                    "Error: '{}' already exists, use --force to override it."
                ).format(output_folder)
            )
            ctx.exit(1)

    staticfolder = os.path.join(output_folder, "static")

    # make the output path
    os.makedirs(output_folder)

    # if has override style copy
    if style:
        shutil.copy(
            style, os.path.join(output_folder, os.path.basename(style))
        )

    shutil.copytree(REVEALJS_FOLDER, os.path.join(staticfolder, "revealjs"))

    # Check for media root
    if not media:
        media = os.path.realpath(os.path.join(path, "media"))
    else:
        media = os.path.realpath(media)

    if not os.path.isdir(media):
        # Running without media folder
        media = None
        click.echo("Media folder not detected, running without media.")
    else:
        shutil.copytree(media, os.path.join(output_folder, "media"))

    # Check for theme root
    if not theme:
        theme = os.path.join(path, "theme")

    if not os.path.isdir(theme):
        # Running without theme folder
        theme = None
        click.echo("Theme not detected, running without custom theme.")
    else:
        shutil.copytree(theme, os.path.join(output_folder, "theme"))

    # Check for configuration file
    if not config:
        config = os.path.join(path, "config.py")

    if not os.path.isfile(config):
        # Running without configuration file
        config = None

        click.echo("Configuration file not detected, running with defaults.")

    click.echo("Generating static presentation...")

    # instantiating revelation app
    app = Revelation(presentation, config, media, theme, style)

    if not os.path.isdir(output_folder):
        os.makedirs(output_folder)

    output_file = os.path.join(output_folder, output_file)

    with open(output_file, "w") as f:
        f.write(app.dispatch_request(None).get_data(as_text=True))

    click.echo(
        "Static presentation generated in {}".format(
            os.path.realpath(output_folder)
        )
    )


@cli.command("start", help="Start the revelation server")
@click.argument("presentation", default=os.getcwd())
@click.option("--port", "-p", default=4000, help="Presentation server port")
@click.option("--config", "-c", default=None, help="Custom configuration file")
@click.option("--media", "-m", default=None, help="Custom media folder")
@click.option("--theme", "-t", default=None, help="Custom theme folder")
@click.option(
    "--style-override-file",
    "-s",
    "style",
    default=None,
    help="Custom css file to override reveal.js styles",
)
@click.option(
    "--debug",
    "-d",
    is_flag=True,
    default=False,
    help="Run the revelation server on debug mode",
)
@click.pass_context
def start(ctx, presentation, port, config, media, theme, style, debug):
    """Start revelation presentation command"""
    # Check if reveal.js is installed
    if not os.path.exists(REVEALJS_FOLDER):
        click.echo("Reveal.js not found, running installation...")
        ctx.invoke(installreveal)

    # Check for presentation file
    if os.path.isfile(presentation):
        path = os.path.dirname(presentation)
    else:
        click.echo("Error: Presentation file not found.")
        ctx.exit(1)

    # Check for style override file
    if style and (not os.path.isfile(style) or not style.endswith(".css")):
        click.echo("Error: Style is not a css file or does not exists.")
        ctx.exit(1)

    # Check for media root
    if not media:
        media = os.path.join(path, "media")

    if not os.path.isdir(media):
        # Running without media folder
        media = None

        click.echo("Media folder not detected, running without media")

    # Check for theme root
    if not theme:
        theme = os.path.join(path, "theme")

    if not os.path.isdir(theme):
        # Running without theme folder
        theme = None

    # Check for configuration file
    if not config:
        config = os.path.join(path, "config.py")

    if not os.path.isfile(config):
        # Running without configuration file
        config = None

        click.echo("Configuration file not detected, running with defaults.")

    click.echo("Starting revelation server...")

    # instantiating revelation app
    app = Revelation(presentation, config, media, theme, style, True)

    if debug:
        app = DebuggedApplication(app)

    PresentationReloader.tracking_path = os.path.abspath(path)

    click.echo("Running at http://localhost:{}".format(port))

    WebSocketServer(
        ("localhost", port),
        Resource(
            [
                ("^/reloader.*", PresentationReloader),
                ("^/.*", DebuggedApplication(app)),
            ]
        ),
    ).serve_forever()



# -*- coding: utf-8 -*-

import shutil
import tempfile
from unittest import TestCase

from revelation.config import Config


class ConfigTestCase(TestCase):
    def setUp(self):
        self.tests_folder = tempfile.mkdtemp()
        _, config_file = tempfile.mkstemp(".py", dir=self.tests_folder)

        with open(config_file, "w") as file:
            file.write(
                "REVEAL_META = {\n"
                '"title": "Test Title",\n'
                '"author": "Test Author",\n'
                '"description": "Test description",\n'
                "}"
            )

        self.config = Config(config_file)

    def tearDown(self):
        shutil.rmtree(self.tests_folder)

    def test_config_extends_dict(self):
        self.assertIsInstance(self.config, dict)

    def test_config_initialize_variables(self):
        self.assertIn("REVEAL_META", self.config)
        self.assertIn("REVEAL_SLIDE_SEPARATOR", self.config)
        self.assertIn("REVEAL_THEME", self.config)
        self.assertIn("REVEAL_CONFIG", self.config)

    def test_config_custom_values(self):
        self.assertDictEqual(
            self.config["REVEAL_META"],
            {
                "title": "Test Title",
                "author": "Test Author",
                "description": "Test description",
            },
        )



# -*- coding: utf-8 -*-

import os
import shutil
import tarfile
import tempfile
import zipfile
from unittest import TestCase

from revelation.utils import extract_file, make_presentation, move_and_replace

try:
    # python 3
    FileNotFoundError
except NameError:
    # legacy python
    FileNotFoundError = IOError


class HelpersTestCase(TestCase):
    def make_src(self, base):
        source = tempfile.mkdtemp(dir=base)
        file_to_replace_on_source = os.path.join(source, "replace.txt")
        fd, file_to_move = tempfile.mkstemp(".txt", dir=source)

        os.close(fd)

        tempfile.mkdtemp(dir=source)

        with open(file_to_replace_on_source, "w") as f_source:
            f_source.write("source")

        return source

    def make_dst(self, base):
        destination = tempfile.mkdtemp(dir=base)
        file_to_replace_on_dest = os.path.join(destination, "replace.txt")

        with open(file_to_replace_on_dest, "w") as f_dest:
            f_dest.write("dest")

        return destination

    def make_tar(self, content, base):
        fd, tar_file = tempfile.mkstemp(".tar.gz", dir=base)

        os.close(fd)

        with tarfile.open(tar_file, "w:gz") as t:
            t.add(content, arcname="tarfolder")

        return tar_file

    def make_zip(self, content, base):
        fd, zip_file = tempfile.mkstemp(".zip", dir=base)

        os.close(fd)

        with zipfile.ZipFile(zip_file, "w") as z:
            z.write(content, arcname="zipfolder")
            for item in os.listdir(content):
                z.write(
                    os.path.join(content, item),
                    arcname=os.path.join("zipfolder", item),
                )

        return zip_file

    def setUp(self):
        self.tests_folder = tempfile.mkdtemp()
        self.source = self.make_src(self.tests_folder)
        self.destination = self.make_dst(self.tests_folder)
        self.tar = self.make_tar(self.source, self.tests_folder)
        self.zip = self.make_zip(self.source, self.tests_folder)
        fd, self.somefile = tempfile.mkstemp(dir=self.tests_folder)

        os.close(fd)

    def tearDown(self):
        shutil.rmtree(self.tests_folder)

    def test_helper_move_and_replace(self):
        src_files = sorted(os.listdir(self.source))

        move_and_replace(self.source, self.destination)

        dst_files = sorted(os.listdir(self.destination))

        with open(os.path.join(self.destination, "replace.txt"), "r") as f:
            file_content = f.read()

        # The moved directory should not exist because it was moved
        self.assertFalse(os.path.exists(self.source))
        # The replaced file should contain the data from the source file
        self.assertEqual(file_content, "source")
        # The moved files from source should be equal to the
        # files on destination directory
        self.assertEqual(src_files, dst_files)

    def test_extract_file_tarfile(self):
        src_files = sorted(os.listdir(self.source))

        extracted = extract_file(self.tar, self.tests_folder)

        extracted_files = sorted(os.listdir(extracted))

        self.assertEqual(extracted_files, src_files)

    def test_extract_file_zipfile(self):
        src_files = sorted(os.listdir(self.source))

        extracted = extract_file(self.zip, self.tests_folder)

        extracted_files = sorted(os.listdir(extracted))

        self.assertEqual(extracted_files, src_files)

    def test_extract_file_on_non_file(self):
        self.assertRaises(FileNotFoundError, extract_file, self.tests_folder)

    def test_extract_file_on_non_tar_or_zip(self):
        self.assertRaises(NotImplementedError, extract_file, self.somefile)

    def test_make_presentation(self):
        path = os.path.join(tempfile.mkdtemp(dir=self.tests_folder), "test")
        media_path = os.path.join(path, "media")
        config_path = os.path.join(path, "config.py")
        presentation_path = os.path.join(path, "slides.md")

        make_presentation(path)

        self.assertTrue(os.path.isdir(path))
        self.assertTrue(os.path.isdir(media_path))
        self.assertTrue(os.path.isfile(config_path))
        self.assertTrue(os.path.isfile(presentation_path))



# -*- coding: utf-8 -*-
"""
Revelation presentation settings

This is an automatic generated template for revelation presentations
with the default options.
"""

REVEAL_META = {
    # Title of the slide
    "title": "The title",
    # Author in the metadata of the slide
    "author": "Some Author",
    # Description in the metadata of the slide
    "description": "Some description",
}

# Reveal markdown slide separator
REVEAL_SLIDE_SEPARATOR = "---"

# Themes
# beige, black, blood, league, moon, night, serif, simple, sky,
# solarized, white
REVEAL_THEME = "black"

REVEAL_CONFIG = {
    # Display controls in the bottom right corner
    "controls": True,
    # Display a presentation progress bar
    "progress": True,
    # Display the page number of the current slide
    "slideNumber": False,
    # Push each slide change to the browser history
    "history": True,
    # Enable keyboard shortcuts for navigation
    "keyboard": True,
    # Enable the slide overview mode
    "overview": True,
    # Vertical centering of slides
    "center": True,
    # Enables touch navigation on devices with touch input
    "touch": True,
    # Loop the presentation
    "loop": False,
    # Change the presentation direction to be RTL
    "rtl": False,
    # Turns fragments on and off globally
    "fragments": True,
    # Flags if the presentation is running in an embedded mode,
    # i.e. contained within a limited portion of the screen
    "embedded": False,
    # Flags if we should show a help overlay when the questionmark
    # key is pressed
    "help": True,
    # Flags if speaker notes should be visible to all viewers
    "showNotes": False,
    # Number of milliseconds between automatically proceeding to the
    # next slide, disabled when set to 0, this value can be overwritten
    # by using a data-autoslide attribute on your slides
    "autoSlide": 0,
    # Stop auto-sliding after user input
    "autoSlideStoppable": True,
    # Enable slide navigation via mouse wheel
    "mouseWheel": False,
    # Hides the address bar on mobile devices
    "hideAddressBar": True,
    # Opens links in an iframe preview overlay
    "previewLinks": False,
    # Transition style
    # default/cube/page/concave/zoom/linear/fade/none
    "transition": "default",
    # Transition speed
    "transitionSpeed": "default",  # default/fast/slow
    # Transition style for full page slide backgrounds
    # default/none/slide/concave/convex/zoom
    "backgroundTransition": "default",
    # Number of slides away from the current that are visible
    "viewDistance": 3,
    # Parallax background image
    # e.g.:
    # "'https://s3.amazonaws.com/hakim-static/reveal-js/reveal-parallax-1.jpg'"
    "parallaxBackgroundImage": "",
    # Parallax background size
    "parallaxBackgroundSize": "",  # CSS syntax, e.g. "2100px 900px"
    # Amount to move parallax background (horizontal and vertical)
    # on slide change
    # Number, e.g. 100
    "parallaxBackgroundHorizontal": "",
    "parallaxBackgroundVertical": "",
}



# -*- coding: utf-8 -*-
"""Revelation configuration handler"""

import os
import types

from werkzeug.utils import import_string

from . import default_config


class Config(dict):
    """
    Class that stores revelation configs thanks to flask Config:

    https://github.com/pallets/flask/blob/master/flask/config.py

    It loads the default_config variables and if a path is passed it also
    loads the external configs
    """

    def __init__(self, filename=None):
        """Initializes the config with the defaults or with custom
        variables from an external file"""
        self.load_from_object(default_config)

        if filename and os.path.isfile(filename):
            self.load_from_pyfile(filename)

    def load_from_object(self, obj):
        """Load the configs from a python object passed
        to the function"""
        if isinstance(obj, str):
            obj = import_string(obj)

        for key in dir(obj):
            if key.isupper():
                self[key] = getattr(obj, key)

    def load_from_pyfile(self, filename):
        """Load the configs from a python file as it was imported"""
        module = types.ModuleType("config")
        module.__file__ = filename

        try:
            with open(filename, mode="rb") as config_file:
                exec(
                    compile(config_file.read(), filename, "exec"),
                    module.__dict__,
                )
        except IOError as error:
            error.strerror = "Unable to load configuration file ({})".format(
                error.strerror
            )

            raise

        self.load_from_object(module)



# -*- coding: utf-8 -*-
"""
Main revelation module

It has the Revelation main class that creates the webserver do run
the presentation
"""

import json
import os
import re

from geventwebsocket import WebSocketApplication
from jinja2 import Environment, PackageLoader, select_autoescape
from watchdog.events import FileSystemEventHandler
from watchdog.observers import Observer
from werkzeug.wrappers import Request, Response
from werkzeug.wsgi import SharedDataMiddleware

from .config import Config


class Revelation(object):
    """
    Main revelation app class that instantiates the server and handles
    the requests
    """

    def __init__(
        self,
        presentation,
        config=None,
        media=None,
        theme=None,
        style=None,
        reloader=False,
    ):
        """
        Initializes the server and creates the environment for the presentation
        """
        self.config = Config(config)
        self.presentation = presentation
        self.reloader = reloader

        shared_data = {
            "/static": os.path.join(os.path.dirname(__file__), "static")
        }

        shared_data.update(self.parse_shared_data(media))
        shared_data.update(self.parse_shared_data(theme))

        if style:
            self.style = os.path.basename(style)
            shared_data.update(self.parse_shared_data(style))
        else:
            self.style = None

        self.wsgi_app = SharedDataMiddleware(self.wsgi_app, shared_data)

    def parse_shared_data(self, shared_root):
        """
        Parse aditional shared_data if it exists
        """
        if shared_root:
            shared_root = os.path.abspath(shared_root)

            if os.path.exists(shared_root):
                shared_url = "/{}".format(os.path.basename(shared_root))

                return {shared_url: shared_root}

        return {}

    def load_slides(self, path, separator):
        """
        Get slides file from the given path, loads it and split into list
        of slides.

        :return: a list of strings with the slides content
        """
        with open(path, "r") as presentation:
            slides = presentation.read()

        return re.split("^{}$".format(separator), slides, flags=re.MULTILINE)

    def get_theme(self, theme):
        reveal_theme = "static/revealjs/css/theme/{}.css".format(theme)
        fullpath_theme = os.path.join(os.path.dirname(__file__), reveal_theme)

        if os.path.isfile(fullpath_theme):
            return reveal_theme

        return theme

    def dispatch_request(self, request):
        env = Environment(
            loader=PackageLoader("revelation", "templates"),
            autoescape=select_autoescape(["html"]),
        )

        context = {
            "meta": self.config.get("REVEAL_META"),
            "slides": self.load_slides(
                self.presentation, self.config.get("REVEAL_SLIDE_SEPARATOR")
            ),
            "config": self.config.get("REVEAL_CONFIG"),
            "theme": self.get_theme(self.config.get("REVEAL_THEME")),
            "style": self.style,
            "reloader": self.reloader,
        }

        template = env.get_template("presentation.html")

        return Response(
            template.render(**context), headers={"content-type": "text/html"}
        )

    def wsgi_app(self, environ, start_response):
        request = Request(environ)
        response = self.dispatch_request(request)

        return response(environ, start_response)

    def __call__(self, environ, start_response):
        return self.wsgi_app(environ, start_response)


class PresentationReloadWebSocketSendEvent(FileSystemEventHandler):
    """
    Handler class to notify throughout the websocket
    when a tracked file changes
    """

    def __init__(self, ws):
        self.ws = ws

    def on_modified(self, event):
        if event.src_path.endswith((".md", ".css")) and not self.ws.closed:
            self.ws.send(
                json.dumps({"msg_type": "message", "message": "reload"})
            )


class PresentationReloader(WebSocketApplication):
    """WebSocket to notify the frontend on file changes"""

    tracking_path = None

    def on_open(self):
        if self.tracking_path:
            event_handler = PresentationReloadWebSocketSendEvent(self.ws)
            self.observer = Observer()
            self.observer.schedule(event_handler, self.tracking_path)
            self.observer.start()

    def on_message(self, message, *args, **kwargs):
        pass

    def on_close(self, reason):
        self.observer.stop()



# -*- coding: utf-8 -*-

import os
import shutil
import tempfile
from unittest import TestCase

from click.testing import CliRunner

from revelation import cli

try:
    # python 3
    from unittest.mock import patch
except ImportError:
    # legacy python
    from mock import patch


class CliTestCase(TestCase):
    def setUp(self):
        self.tests_folder = tempfile.mkdtemp()

    def tearDown(self):
        shutil.rmtree(self.tests_folder)

    def test_mkpresentation(self):
        presentation_folder = os.path.join(
            self.tests_folder, "test_mkpresentation"
        )
        presentation_file = os.path.join(presentation_folder, "slides.md")
        media_folder = os.path.join(presentation_folder, "media")
        config_file = os.path.join(presentation_folder, "config.py")

        runner = CliRunner()
        result = runner.invoke(cli.mkpresentation, [presentation_folder])

        self.assertEqual(result.exit_code, 0)
        self.assertTrue(os.path.isdir(presentation_folder))
        self.assertTrue(os.path.isfile(presentation_file))
        self.assertTrue(os.path.isdir(media_folder))
        self.assertTrue(os.path.isfile(config_file))

    def test_mkpresentation_already_exists(self):
        presentation = tempfile.mkdtemp(dir=self.tests_folder)

        runner = CliRunner()
        result = runner.invoke(cli.mkpresentation, [presentation])

        self.assertEqual(result.exit_code, 1)
        self.assertEqual(
            result.output,
            "Error: '{}' already exists.\n".format(
                os.path.abspath(presentation)
            ),
        )

    def test_mkstatic(self):
        base_folder = tempfile.mkdtemp(dir=self.tests_folder)
        _, presentation_file = tempfile.mkstemp(
            ".md", "slides", base_folder, "# Test\n"
        )
        output_folder = os.path.join(base_folder, "output")
        index_file = os.path.join(output_folder, "index.html")
        static_folder = os.path.join(output_folder, "static")

        runner = CliRunner()
        result = runner.invoke(
            cli.mkstatic, [presentation_file, "-o", output_folder]
        )

        self.assertEqual(result.exit_code, 0)
        self.assertTrue(os.path.isdir(output_folder))
        self.assertTrue(os.path.isfile(index_file))
        self.assertTrue(os.path.isdir(static_folder))

    def test_mkstatic_output_already_exists_file(self):
        base_folder = tempfile.mkdtemp(dir=self.tests_folder)
        _, presentation_file = tempfile.mkstemp(
            ".md", "slides", base_folder, "# Test\n"
        )
        _, output = tempfile.mkstemp(dir=base_folder)

        runner = CliRunner()
        result = runner.invoke(cli.mkstatic, [presentation_file, "-o", output])

        self.assertEqual(result.exit_code, 1)
        self.assertEqual(
            result.output,
            "Error: '{}' already exists and is a file.\n".format(
                os.path.abspath(output)
            ),
        )

    def test_mkstatic_output_already_exists_folder(self):
        base_folder = tempfile.mkdtemp(dir=self.tests_folder)
        _, presentation_file = tempfile.mkstemp(
            ".md", "slides", base_folder, "# Test\n"
        )
        output = tempfile.mkdtemp(dir=base_folder)

        runner = CliRunner()
        result = runner.invoke(cli.mkstatic, [presentation_file, "-o", output])

        self.assertEqual(result.exit_code, 1)
        self.assertEqual(
            result.output,
            "Error: '{}' already exists, use --force to override it.\n".format(
                os.path.abspath(output)
            ),
        )

    def test_mkstatic_presentation_not_found(self):
        base_folder = tempfile.mkdtemp(dir=self.tests_folder)
        presentation = os.path.join(base_folder, "notfound")

        runner = CliRunner()
        result = runner.invoke(cli.mkstatic, [presentation])

        self.assertEqual(result.exit_code, 1)
        self.assertEqual(
            result.output, "Error: Presentation file not found.\n"
        )

    def test_mkstatic_style_not_file(self):
        base_folder = tempfile.mkdtemp(dir=self.tests_folder)
        _, presentation_file = tempfile.mkstemp(
            ".md", "slides", base_folder, "# Test\n"
        )
        style = tempfile.mkdtemp(".css", dir=base_folder)

        runner = CliRunner()
        result = runner.invoke(cli.mkstatic, [presentation_file, "-s", style])

        self.assertEqual(result.exit_code, 1)
        self.assertEqual(
            result.output,
            "Error: Style is not a css file or does not exists.\n",
        )

    def test_mkstatic_style_not_css(self):
        base_folder = tempfile.mkdtemp(dir=self.tests_folder)
        _, presentation_file = tempfile.mkstemp(
            ".md", "slides", base_folder, "# Test\n"
        )
        _, style = tempfile.mkstemp(".wrong", dir=base_folder)

        runner = CliRunner()
        result = runner.invoke(cli.mkstatic, [presentation_file, "-s", style])

        self.assertEqual(result.exit_code, 1)
        self.assertEqual(
            result.output,
            "Error: Style is not a css file or does not exists.\n",
        )

    @patch("revelation.cli.WebSocketServer")
    def test_start(self, websocketserver_patch):
        base_folder = tempfile.mkdtemp(dir=self.tests_folder)
        _, presentation_file = tempfile.mkstemp(
            ".md", "slides", base_folder, "# Test\n"
        )

        runner = CliRunner()
        runner.invoke(cli.start, [presentation_file])

        self.assertTrue(websocketserver_patch.called)

    def test_start_presentation_not_found(self):
        base_folder = tempfile.mkdtemp(dir=self.tests_folder)
        presentation = os.path.join(base_folder, "notfound")

        runner = CliRunner()
        result = runner.invoke(cli.start, [presentation])

        self.assertEqual(result.exit_code, 1)
        self.assertEqual(
            result.output, "Error: Presentation file not found.\n"
        )

    def test_start_style_not_file(self):
        base_folder = tempfile.mkdtemp(dir=self.tests_folder)
        _, presentation_file = tempfile.mkstemp(
            ".md", "slides", base_folder, "# Test\n"
        )
        style = tempfile.mkdtemp(".css", dir=base_folder)

        runner = CliRunner()
        result = runner.invoke(cli.start, [presentation_file, "-s", style])

        self.assertEqual(result.exit_code, 1)
        self.assertEqual(
            result.output,
            "Error: Style is not a css file or does not exists.\n",
        )

    def test_start_style_not_css(self):
        base_folder = tempfile.mkdtemp(dir=self.tests_folder)
        _, presentation_file = tempfile.mkstemp(
            ".md", "slides", base_folder, "# Test\n"
        )
        _, style = tempfile.mkstemp(".wrong", dir=base_folder)

        runner = CliRunner()
        result = runner.invoke(cli.start, [presentation_file, "-s", style])

        self.assertEqual(result.exit_code, 1)
        self.assertEqual(
            result.output,
            "Error: Style is not a css file or does not exists.\n",
        )



# -*- coding: utf-8 -*-
"""Utility tools used by revelation"""

import os
import shutil
import tarfile
import zipfile

try:
    # python 3
    from urllib.request import urlretrieve
except ImportError:
    # legacy python
    from urllib import urlretrieve

    FileNotFoundError = IOError

from . import default_config

REVEAL_URL = "https://github.com/hakimel/reveal.js/archive/3.6.0.tar.gz"


def make_presentation(presentation_path):
    """
    Make a new presentation boilerplate code given a presentation_path
    """
    name = os.path.basename(presentation_path)
    # Presentation dir
    os.mkdir(presentation_path)
    # Media dir
    os.mkdir(os.path.join(presentation_path, "media"))
    # Config file
    config_file = os.path.join(
        os.path.dirname(default_config.__file__), "default_config.py"
    )
    shutil.copy(config_file, os.path.join(presentation_path, "config.py"))
    # Slide file
    with open(os.path.join(presentation_path, "slides.md"), "w") as f:
        f.write(
            "# {0}\n\nStart from here!".format(
                name.replace("_", " ").replace("-", " ").title()
            )
        )


def download_reveal(url=None):
    """
    Download reveal.js installation files
    """
    if not url:
        url = REVEAL_URL

    try:
        return urlretrieve(url)
    except Exception:
        raise


def move_and_replace(src, dst):
    """
    Helper function used to move files from one place to another,
    creating os replacing them if needed

    :param src: source directory
    :param dst: destination directory
    """

    src = os.path.abspath(src)
    dst = os.path.abspath(dst)

    for src_dir, _, files in os.walk(src):
        # using os walk to navigate through the directory tree
        # keep te dir structure by replacing the source root to
        # the destination on walked path
        dst_dir = src_dir.replace(src, dst)

        if not os.path.exists(dst_dir):
            # to prevent copy from failing, create the not existing dirs
            os.makedirs(dst_dir)

        for file_ in files:
            src_file = os.path.join(src_dir, file_)
            dst_file = os.path.join(dst_dir, file_)

            if os.path.exists(dst_file):
                os.remove(dst_file)  # to copy not fail, create existing files

            shutil.move(src_file, dst_dir)  # move the files

    shutil.rmtree(src)  # remove the dir structure from the source


def extract_file(compressed_file, path="."):
    """Extract function to extract from zip or tar file"""
    if os.path.isfile(compressed_file):
        if tarfile.is_tarfile(compressed_file):
            with tarfile.open(compressed_file, "r:gz") as tfile:
                basename = tfile.members[0].name
                tfile.extractall(path + "/")
        elif zipfile.is_zipfile(compressed_file):
            with zipfile.ZipFile(compressed_file, "r") as zfile:
                basename = zfile.namelist()[0]
                zfile.extractall(path)
        else:
            raise NotImplementedError("File type not supported")
    else:
        raise FileNotFoundError(
            "{0} is not a valid file".format(compressed_file)
        )

    return os.path.abspath(os.path.join(path, basename))



# -*- coding: utf-8 -*-

import os
import shutil
import tempfile
from unittest import TestCase

from werkzeug.test import Client
from werkzeug.wrappers import BaseResponse

from revelation import Revelation


class RevelationTestCase(TestCase):
    def setUp(self):
        self.tests_folder = tempfile.mkdtemp()
        self.media = tempfile.mkdtemp(dir=self.tests_folder)
        _, self.slide = tempfile.mkstemp(".md", dir=self.tests_folder)

        with open(self.slide, "w") as file:
            file = file.write("# Pag1\n---\n# Pag2")

        self.app = Revelation(self.slide, media=self.media)

    def tearDown(self):
        shutil.rmtree(self.tests_folder)

    def test_parse_shared_data_empty(self):
        shared_data_config = self.app.parse_shared_data(None)

        self.assertDictEqual(shared_data_config, {})

    def test_parse_shared_data(self):
        shared_data_config = self.app.parse_shared_data(self.media)

        self.assertDictEqual(
            shared_data_config,
            {"/{}".format(os.path.basename(self.media)): self.media},
        )

    def test_load_slides(self):
        slides = self.app.load_slides(self.slide, "---")

        self.assertListEqual(slides, ["# Pag1\n", "\n# Pag2"])

    def test_client_request_ok(self):
        client = Client(self.app, BaseResponse)
        response = client.get("/")
        self.assertEqual(response.status, "200 OK")
        self.assertEqual(response.headers.get("Content-Type"), "text/html")

    def test_client_with_reload(self):
        app = Revelation(self.slide, media=self.media, reloader=True)
        client = Client(app, BaseResponse)
        response = client.get("/")
        self.assertIn("reloader", response.data.decode("utf8"))

    def test_client_without_reload(self):
        app = Revelation(self.slide, media=self.media, reloader=False)
        client = Client(app, BaseResponse)
        response = client.get("/")
        self.assertNotIn("reloader", response.data.decode("utf8"))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""revelation setup file"""

import os
import re

from setuptools import find_packages, setup

PACKAGE = "revelation"
REQUIREMENTS = [
    "Jinja2==2.10",
    "Werkzeug==0.14.1",
    "click==6.7",
    "gevent-websocket==0.10.1",
    "gevent==1.3.4",
    "watchdog==0.8.3",
]
TEST_REQUIREMENTS = [
    "coverage==4.5.1",
    "coveralls==1.3.0",
    "flake8==3.5.0",
    "mock",
    "nose==1.3.7",
]

with open("README.md", "r") as f:
    README = f.read()

with open(os.path.join(PACKAGE, "__init__.py")) as init_file:
    INIT = init_file.read()

VERSION = re.search(
    "^__version__ = ['\"]([^'\"]+)['\"]", INIT, re.MULTILINE
).group(1)
AUTHOR = re.search(
    "^__author__ = ['\"]([^'\"]+)['\"]", INIT, re.MULTILINE
).group(1)
EMAIL = re.search(
    "^__email__ = ['\"]([^'\"]+)['\"]", INIT, re.MULTILINE
).group(1)

setup(
    name=PACKAGE,
    version=VERSION,
    description="Make awesome reveal.js presentations with revelation",
    long_description=README,
    long_description_content_type="text/markdown",
    author=AUTHOR,
    author_email=EMAIL,
    url="https://github.com/humrochagf/revelation",
    license="MIT",
    packages=find_packages(),
    package_data={PACKAGE: ["templates/presentation.html"]},
    zip_safe=False,
    install_requires=REQUIREMENTS,
    entry_points=dict(console_scripts=["revelation=revelation.cli:cli"]),
    platforms="any",
    keywords="presentation slides reveal.js markdown",
    classifiers=[
        "Environment :: Console",
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Topic :: Multimedia :: Graphics :: Presentation",
        "Topic :: Text Processing :: Markup :: HTML",
    ],
    test_suite="tests",
    tests_require=TEST_REQUIREMENTS,
    extras_require={"test": TEST_REQUIREMENTS},
)
